package com.muyu.etl.data.access.data.access.client.config;

import com.muyu.common.core.domain.Result;
import com.muyu.etl.data.access.data.access.client.basic.DataSourceConfig;
import com.muyu.etl.data.access.domain.resp.DataSourceResp;
import com.muyu.etl.data.access.remote.RemoteDataSourceService;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Author WangXin
 * @Data 2024/9/6
 * @Description 数据连接池
 * @Version 1.0.0
 */
@Log4j2
@Component
public class DatabaseConnectionPool {

    /**
     * 数据源信息
     */
    private final ConcurrentHashMap<Long, HikariDataSource> map = new ConcurrentHashMap<>();

    @Autowired
    private DataSourceConfig dataSourceConfig;

    /**
     * 初始化连接池
     *
     * @param dataSourceResp 连接池数据
     */
    public void initialize (DataSourceResp dataSourceResp) {
        HikariConfig config = new HikariConfig();
        config.setDriverClassName(dataSourceResp.getDriverName());
        config.setJdbcUrl("jdbc:mysql://" + dataSourceResp.getDatabaseUrl() + ":" + dataSourceResp.getDatabasePost() + "/" + dataSourceResp.getDatabaseName() + "?" + dataSourceResp.getDatabaseFormName());
        config.setUsername(dataSourceResp.getUserName());
        config.setPassword(dataSourceResp.getUserPwd());
        config.setMaximumPoolSize(dataSourceResp.getInitCount());
        config.addDataSourceProperty("cachePrepStmts", "true");
        map.putIfAbsent(dataSourceResp.getId(), new HikariDataSource(config));
    }


    /**
     * 初始化数据库
     */
    public void init () {
        log.info("初始化连接池开始……………………");
        long startTime = System.currentTimeMillis();
        List<DataSourceResp> dataSourceRespList = dataSourceConfig.getIsUseDataSourceList();
        log.info("查询所有可用连接池配置：{}个，连接池名称：{}", dataSourceRespList.size(), dataSourceRespList.stream().map(DataSourceResp::getDatabaseName).toList());
        for (DataSourceResp dataSourceResp : dataSourceRespList) {
            log.info("开始初始化：[{}]连接", dataSourceResp.getDatabaseName());
            initialize(dataSourceResp);
            log.info("完成初始化：[{}]连接", dataSourceResp.getDatabaseName());
        }
        log.info("初始化连接池结束共计耗时:{}MS……………………", System.currentTimeMillis() - startTime);
    }

    /**
     * 获取数据连接对象
     *
     * @param id 数据库id
     *
     * @return HikariDataSource数据库连接所需对象
     */
    public HikariDataSource get (Long id) {
        if (map.get(id) == null) {
            throw new RuntimeException("请重新初始化数据源!!!");
        }
        return map.get(id);
    }

    /**
     * 归还连接
     *
     * @param connection 连接对象
     */
    public void returnConnection (Connection connection, HikariDataSource hikariDataSource) {
        try {
            if (connection != null) {
                connection.close();
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    public Connection getConn (HikariDataSource hikariDataSource) {
        Connection conn = null;
        try {
            conn = hikariDataSource.getConnection();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return conn;
    }
}
