package com.muyu.etl.data.access.controller;


import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.etl.data.access.domain.DataSource;
import com.muyu.etl.data.access.domain.req.DataSourceReq;
import com.muyu.etl.data.access.domain.req.DataSourceUpdReq;
import com.muyu.etl.data.access.domain.resp.DataSourceResp;
import com.muyu.etl.data.access.service.DataSourceService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.log4j.Log4j2;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;


/**
 * @version 1.0
 * @Author xie ya ru
 * @Date 2024/8/19 19:46
 * @注释
*/
@Log4j2
@RestController
@RequestMapping("/dataSource")
@Tag(name = "数据源控制层",description = "进行对数据源连接,查询等相关操作")
public class DataSourceController extends BaseController {


    @Resource
    private DataSourceService dataSourceService;

    /**
     * 查看所有数据源信息
     * @param dataSourceReq 请求参数
     * @return List<DataSourceResp>
     */
    @RequiresPermissions(value = "data:datasource:list")
    @Operation(summary = "查看所有数据源信息",description = "根据数据源类型,数据源名称等进行查询数据源信息")
    @PostMapping("/list")
    public Result<TableDataInfo<DataSourceResp>> selectList(@Validated @RequestBody DataSourceReq dataSourceReq) {
        startPage();
        List<DataSourceResp> dataSourceRespList = dataSourceService.selectList(dataSourceReq);
        return getDataTable(dataSourceRespList);

    }

    @GetMapping("/is-use-datasource")
    public Result<List<DataSourceResp>> getIsUseDataSourceList(){
        return Result.success(dataSourceService.getIsUseDataSourceConfig());
    }

    /**
     * 添加数据源
     * @param dataSource 添加数据源详细信息
     * @return 添加结果
     */
    @Log(title = "数据接入", businessType = BusinessType.INSERT)
    @RequiresPermissions(value = "data:datasource:list")
    @Operation(summary = "数据源信息添加", description = "添加数据源的信息,才能连接进行访问获取数据")
    @PostMapping
    public Result<?> save(@Validated @RequestBody DataSource dataSource) {
        dataSourceService.save(dataSource);
        return Result.success(null,"操作成功");
    }

    /**
     * 删除数据源信息
     * @param id 请求参数
     * @return 删除结果
     */
    @Log(title = "数据接入", businessType = BusinessType.DELETE)
    @RequiresPermissions(value = "data:datasource:list")
    @Operation(summary = "删除数据源信息", description = "根据数据源ID删除数据源信息")
    @DeleteMapping("/{id}")
    public Result<?> removeById(
            @Schema(title="数据源ID",type = "Long",defaultValue = "1",description = "删除数据源信息的唯一条件")
            @Validated @PathVariable("id") Long id) {
        dataSourceService.removeById(id);
        return Result.success(null,"操作成功");
    }

    /**
     * 修改数据源信息
     * @param id  dataSourceUpdReq
     * @return 修改结果
     */
    @Log(title = "数据接入", businessType = BusinessType.UPDATE)
    @RequiresPermissions(value = "data:datasource:list")
    @Operation(summary = "修改数据源信息", description = "根据数据源ID修改数据源信息")
    @PutMapping("/{id}")
    public Result<?> updateById(
            @Schema(title="数据源ID",type = "Long",defaultValue = "1",description = "修改数据源信息的唯一条件")
            @Validated @PathVariable("id") Long id,
            @RequestBody DataSourceUpdReq dataSourceUpdReq) {
        System.out.println(dataSourceUpdReq);
            dataSourceService.updateById(DataSource.updBuild(id,dataSourceUpdReq));
        return Result.success(null,"操作成功");
    }

    /**
     * 根据数据源ID查询数据源信息
     * @param id 请求参数
     * @return 返回根据数据源ID查询出的数据源信息
     */
    @RequiresPermissions(value = "data:datasource:list")
    @Operation(summary = "根据数据源ID查询数据源信息", description = "根据数据源ID查询数据源信息")
    @GetMapping("/{id}")
    public Result<DataSourceResp> getById(

            @Schema(title="数据源ID",type = "Long",defaultValue = "1",description = "查询数据源信息的唯一条件")
            @Validated @PathVariable("id") Long id) {
        return Result.success(DataSourceResp.dataSourceRespBuild(dataSourceService.getById(id)),"操作成功");
    }


    /**
     * 测试连接数据库
     * @param dataSource
     * @return
     */
//    @RequiresPermissions(value = "data:datasource:list")
    @Operation(summary = "连接数据库",description = "测试数据库连接")
    @PostMapping("/testCoon")
    public Result<Integer> testCoon(@RequestBody DataSource dataSource) {
        int i= dataSourceService.connectionMysql(dataSource);
        return i > 0 ? Result.success(null,"连接成功") : Result.error(null,"操作失败");
    }

    /**
     * 同步数据结构
     * @param dataSource
     * @return 同步结果
     */
    @Operation(summary = "根据连接信息同步数据结构",description = "根据连接信息同步数据结构")
    @PostMapping("/queryData")
    public Result syncAssetStructure(@RequestBody DataSource dataSource){
        Integer i = dataSourceService.syncAssetStructure(dataSource);
        return i > 0  ? Result.success(null,"数据同步成功") :Result.error("数据同步失败");
    }





}
