package com.muyu.etl.data.access.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.LoginUser;
import com.muyu.common.system.domain.SysUser;
import com.muyu.etl.data.access.domain.EtlTableUserPermission;
import com.muyu.etl.data.access.domain.TableInfo;
import com.muyu.etl.data.access.domain.resp.TableInfoResp;
import com.muyu.etl.data.access.service.EtlTableUserPermissionService;
import com.muyu.etl.data.access.service.TableInfoService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.log4j.Log4j2;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashSet;
import java.util.List;


/**
 * @version 1.0
 * @Author xie ya ru
 * @Date 2024/8/28 18:58
 * @注释
 */
@Log4j2
@RestController
@RequestMapping("/tableInfo")
@Tag(name = "数据库对应表的查询",description = "对自由数据数据库对应表的查询")
public class TableInfoController extends BaseController {

    @Resource
    private TableInfoService tableInfoService;

    @Resource
    private EtlTableUserPermissionService etlTableUserPermissionService;



    /**
     * 查询数据库/表 级联查询用于数据资产授权
     * 自己授权用
     * @return 返回结果
     */

    @Operation(summary = "查询数据库/表 级联查询用于数据资产授权" +
            "自己授权用" ,
            description = "查询数据库/表 级联查询用于数据资产授权" +
                    "自己授权用")
    @GetMapping("/findByTableName")
    public Result<List<TableInfoResp>> findByTableName(){
        //从服务中获取TableInfo对象的列表
        List<TableInfo> list = tableInfoService.list();
        //使用Stream API来处理列表，创建一个新的列表respList
        List<TableInfoResp> respList = list.stream()
                //筛选出parentId为0的TableInfo对象，通常表示这些是顶级条目
                .filter(tableInfo -> tableInfo.getParentId()==0)
                //将筛选出的TableInfo对象转换为响应对象TableInfoListResp
                .map(tableInfo -> {
                    //将TableInfo对象转换为TableInfoListResp对象
                    TableInfoResp tableInfoResp = TableInfo.toTableInfoListResp(tableInfo);
                    //递归地为每个TableInfoListResp对象设置子对象列表
                    tableInfoResp.setChildren(getAssetChildren(tableInfo, list));
                    return tableInfoResp;
                })
                //将Stream转换为List
                .toList();
        //注意：toList()是Java 16及以上版本中的方法，如果是Java 8或Java 11，应使用collect(Collectors.toList())

        //构建并返回一个成功的响应对象，包含处理后的列表
        return Result.success(respList);
    }


    @Operation(summary = "根据主键ID查询数据",description = "根据主键ID查询数据")
    @GetMapping("/getTableInfoDetails/{ids}")
    Result<List<TableInfo>> getTableInfoDetails(@PathVariable("ids") Integer[] ids){
        LambdaQueryWrapper<TableInfo> wrapper = new LambdaQueryWrapper<>();
        wrapper.in(TableInfo::getId, (Object[]) ids);
        return Result.success(tableInfoService.list(wrapper));
    }

    /**
     * 效验空 资产授权
     * 查询数据库/表 级联查询用于数据资产授权
     * 自己授权用 任务调用接口
     * @param tableInfo 实体类
     * @param list 结果集
     * @return 返回结果
     */
    private static List<TableInfoResp> getAssetChildren(TableInfo tableInfo, List<TableInfo> list) {
        //使用Stream API处理list集合
        return list.stream()//将list转换为Stream
                //使用filter方法筛选出那些parentId等于tableInfo.getId()的TableInfo对象
                .filter(tableInfo1 -> tableInfo1.getParentId().equals(tableInfo.getId()))
                //使用map方法将筛选出的TableInfo对象转换为TableInfoListResp对象
                .map(
                        tableInfo2 -> TableInfo.toTableInfoListResp(tableInfo2))
                //使用toList方法将结果收集到一个新的列表中
                .toList();
        //目前用的Java 17
        //toList()是Java 16及以上版本中的方法，如果是Java 8或Java 11，应使用collect(Collectors.toList())
    }




    /**
     * 过滤级联查询方法，资产授权，获取当前用户的信息
     * 资产展示调用方法
     * @return 返回结果
     */
    @Operation(summary = "过滤级联查询方法，资产授权，获取当前用户的信息" ,
            description = "过滤级联查询方法，资产授权，获取当前用户的信息内容 赋予权限")
    @GetMapping("findAssetByTableName")
    public Result findAsserByTableName(){
//        //从服务中获取TableInfo对象的列表
//        List<TableInfo> list = tableInfoService.list();
        //使用安全管理工具获取当前登录的用户对象
        LoginUser loginUser = SecurityUtils.getLoginUser();
        //从登录用户对象中获取系统用户对象
        SysUser sysUser = loginUser.getSysUser();
        //从系统用户对象中获取用户ID
        Long userId = sysUser.getUserId();
        //从系统用户对象中获取部门ID
        Long deptId = sysUser.getDeptId();

        // 调用Service服务的findTableIdAndBasicIdByUserId方法
        // 传入用户ID，获取该用户授权的所有表ID和基础ID
        List<EtlTableUserPermission> idByUserId = etlTableUserPermissionService.findTableIdAndBasicIdByUserId(userId);

        // 调用Service服务的findTableIdAndBasicIdByDeptId方法
        // 传入部门ID，获取该部门下所有授权的表ID和基础ID
        List<EtlTableUserPermission> idByDeptId = etlTableUserPermissionService.findTableIdAndBasicIdByDeptId(deptId);


        //创建一个HashSet集合，用于存储唯一的TableInfo对象
        HashSet<TableInfo> hashSet = new HashSet<>();
        //遍历idByUserId列表，这个列表可能包含了用户授权的资产信息 findBasicIdAndTableId
        for (EtlTableUserPermission etlTableUserPermission : idByUserId) {
            //获取资产信息中的基本ID
            Long basicId = etlTableUserPermission.getBasicId();
            //获取资产信息中的表ID
            Long tableId = etlTableUserPermission.getTableId();
            //如果基本原来表的ID不为空，则进行查询
            if (null != basicId) {
                //创建一个LambdaQueryWrapper，用于构建查询条件
                LambdaQueryWrapper<TableInfo> queryWrapper = new LambdaQueryWrapper<>();
                //添加查询条件，筛选出基本ID等于当前基本ID的TableInfo对象
                queryWrapper.eq(TableInfo::getBasicId, basicId);
                //调用服务方法，获取符合条件的TableInfo列表
                List<TableInfo> tableInfoBasicIdList = tableInfoService.list(queryWrapper);
                //将查询结果添加到HashSet集合中
                hashSet.addAll(tableInfoBasicIdList);
            }
            //如果当前表ID主键不为空，则进行查询
            if (null != tableId) {
                //调用服务方法，根据表ID获取TableInfo对象
                TableInfo tableInfoById = tableInfoService.getById(tableId);
                //将查询到的TableInfo对象添加到HashSet集合中
                hashSet.add(tableInfoById);
                //获取TableInfo对象的父ID
                Long parentId = tableInfoById.getParentId();
                //如果父ID不为空，则根据父ID查询父TableInfo对象
                TableInfo tableInfoByIdParentId = tableInfoService.getById(parentId);
                //将父TableInfo对象添加到HashSet集合中
                hashSet.add(tableInfoByIdParentId);
            }
        }

        //遍历包含用户授权信息的集合，这个集合可能是根据部门ID获取的
        for (EtlTableUserPermission assetImpowerResp : idByDeptId) {
            //从当前授权对象中获取基本ID
            Long basicId = assetImpowerResp.getBasicId();
            //从当前授权对象中获取表ID
            Long tableId = assetImpowerResp.getTableId();
            //如果基本ID不为空，则进行查询
            if(null!=basicId){
                //创建一个LambdaQueryWrapper，用于构建查询条件
                LambdaQueryWrapper<TableInfo> queryWrapper = new LambdaQueryWrapper<>();
                //添加查询条件，筛选出基本ID等于当前基本ID的TableInfo对象
                queryWrapper.eq(TableInfo::getBasicId, basicId);
                //调用服务方法，获取符合条件的TableInfo列表
                List<TableInfo> tableInfoBasicIdList = tableInfoService.list(queryWrapper);
                //将查询结果添加到HashSet集合中
                hashSet.addAll(tableInfoBasicIdList);
            }
            //如果表ID不为空，则进行查询
            if(null!=tableId){
                //调用服务方法，根据表ID获取TableInfo对象
                TableInfo tableInfoById = tableInfoService.getById(tableId);
                //将查询到的TableInfo对象添加到HashSet集合中
                hashSet.add(tableInfoById);
                //获取TableInfo对象的父ID
                Long parentId = tableInfoById.getParentId();
                //如果父ID不为空，则根据父ID查询父TableInfo对象
                TableInfo tableInfoByIdParentId = tableInfoService.getById(parentId);
                //将父TableInfo对象添加到HashSet集合中
                hashSet.add(tableInfoByIdParentId);
            }
        }
        //使用Stream API来处理列表
        List<TableInfoResp> respHashSet = hashSet.stream()
                //筛选出parentId为0的TableInfo对象
                .filter(tableInfo -> tableInfo.getParentId()==0)
                //将筛选出的TableInfo对象转换为响应对象TableInfoListResp
                .map(tableInfo -> {
                    // 将TableInfo对象转换为TableInfoListResp对象
                    TableInfoResp tableInfoResp = TableInfo.toTableInfoListResp(tableInfo);
                    //递归地为每个TableInfoListResp对象设置子对象列表
                    tableInfoResp.setChildren(getChildren(tableInfo, hashSet));
                    return tableInfoResp;
                })
                //将Stream转换回List
                .toList();
        //返回成功的结果，包含处理后的列表
        return Result.success(respHashSet);
    }

    /**
     * 效验空 资产授权
     * 查询数据库/表 级联查询用于数据资产授权
     * 自己授权用 任务调用接口
     * @param tableInfo 实体类
     * @param set 结果集
     * @return 返回结果
     */
    private static List<TableInfoResp> getChildren(TableInfo tableInfo, HashSet<TableInfo> set) {
        //使用Stream API处理list集合
        return set.stream()//将list转换为Stream
                //使用filter方法筛选出那些parentId等于tableInfo.getId()的TableInfo对象
                .filter(tableInfo1 -> tableInfo1.getParentId().equals(tableInfo.getId()))
                //使用map方法将筛选出的TableInfo对象转换为TableInfoListResp对象
                .map(
                        tableInfo2 -> TableInfo.toTableInfoListResp(tableInfo2))
                //使用toList方法将结果收集到一个新的列表中
                .toList();
        //目前用的Java 17
        //toList()是Java 16及以上版本中的方法，如果是Java 8或Java 11，应使用collect(Collectors.toList())
    }


}
