package com.muyu.etl.data.access.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dtflys.forest.utils.StringUtils;
import com.muyu.etl.data.access.data.access.client.config.DatabaseConnectionPool;
import com.muyu.etl.data.access.mapper.DataSourceMapper;
import com.muyu.etl.data.access.service.DataAnalysisService;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.etl.data.access.domain.DataAnalysis;
import com.muyu.etl.data.access.domain.DataSource;
import com.muyu.etl.data.access.domain.TableInfo;
import com.muyu.etl.data.access.domain.req.DataSourceReq;
import com.muyu.etl.data.access.domain.resp.DataSourceResp;
import com.muyu.etl.data.access.pool.RedisPool;
import com.muyu.etl.data.access.service.DataSourceService;
import com.muyu.etl.data.access.service.TableInfoService;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import redis.clients.jedis.Jedis;

import javax.annotation.Resource;
import java.sql.*;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import static com.muyu.etl.data.access.contains.Constants.*;
import static com.muyu.etl.data.access.pool.base.BaseConfig.SHOW_TABLE_STATUS;

/**
 * @version 1.0
 * @Author xie ya ru
 * @Date 2024/8/19 19:42
 * @注释
 */
@Log4j2
@Service
public class DataSourceServiceImpl extends ServiceImpl<DataSourceMapper, DataSource> implements DataSourceService {



    @Resource
    private TableInfoService tableInfoService;
    @Resource
    private DataAnalysisService dataAnalysisService;

    @Autowired
    private DatabaseConnectionPool databaseConnectionPool;

    /**
     * 添加数据来源
     * @param dataSource
     * @return
     */
    @Override
    public boolean save(DataSource dataSource) {

        String type = dataSource.getDatabaseType();

        if (type.equals(MYSQL)){
            dataSource.setDriverName(DRIVERNAME);
        }

        return super.save(dataSource);
    }


    @Override
    public List<DataSourceResp> selectList(DataSourceReq dataSourceReq) {
        LambdaQueryWrapper<DataSource> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.like(StringUtils.isNotBlank(dataSourceReq.getDatabaseName()), DataSource::getDatabaseName, dataSourceReq.getDatabaseName());
        lambdaQueryWrapper.like(StringUtils.isNotBlank(dataSourceReq.getDatabaseType()), DataSource::getDatabaseName, dataSourceReq.getDatabaseType());
        lambdaQueryWrapper.like(StringUtils.isNotBlank(dataSourceReq.getDatabaseUrl()), DataSource::getDatabaseUrl, dataSourceReq.getDatabaseUrl());
        lambdaQueryWrapper.eq(StringUtils.isNotBlank(dataSourceReq.getStatus()), DataSource::getDatabaseName, dataSourceReq.getStatus());
        List<DataSource> list = this.list(lambdaQueryWrapper);
        return list.stream()
                .map(DataSourceResp::dataSourceRespBuild)
                .toList();
    }

    /**
     * 查询数据库的字段名
     * @param dataSource
     * @return
     */
    @Override
    public Integer connectionMysql(DataSource dataSource) {
        if (dataSource.getDatabaseType().equals(MYSQL)){
            databaseConnectionPool.initialize(DataSourceResp.
                    builder()
                    .id(dataSource.getId())
                    .driverName(DRIVERNAME)
                    .databaseUrl(dataSource.getDatabaseUrl())
                    .databasePost(dataSource.getDatabasePost())
                    .databaseName(dataSource.getDatabaseName())
                    .databaseFormName(dataSource.getDatabaseFormName())
                    .userName(dataSource.getUserName())
                    .userPwd(dataSource.getUserPwd())
                    .initCount(dataSource.getInitCount())
                    .build());
            return 1;
        }else if (dataSource.getDatabaseType().equals(REDIS)){
            RedisPool redisPool = new RedisPool(dataSource);
            redisPool.init();
            Jedis conn = redisPool.getConn();
            redisPool.replease(conn);
            redisPool.closeConn();
            return 1;
        }
        return 0;
    }




    @Override
    public Integer syncAssetStructure(DataSource dataSource) {
        HikariDataSource hikariDataSource = databaseConnectionPool.get(dataSource.getId());
        try {
            //检查etlDataScore对象的类型是否为MYSQL
            if (dataSource.getDatabaseType().equals(MYSQL)) {
                //从连接池中获取一个数据库连接
                Connection conn = hikariDataSource.getConnection();
                //构建一个新的TableInfo对象，用于插入或更新数据库中的table_info表
                TableInfo tableInfoInsert = TableInfo.builder()
                        .basicId(dataSource.getId())
                        .parentId(PARENTID)
                        .tableRemark("")// 初始化表备注为空
                        .isCenter("Y")// 可能表示中心数据库的标志
                        .type("dataSource")// 表示数据库
                        .tableName(dataSource.getDatabaseName())// 数据库名作为表名
                        .createBy(SecurityUtils.getUsername())// 获取当前用户名作为创建者
                        .createTime(new Date())// 设置创建时间为当前时间
                        .build();

                //添加数据库table_info表
                //插入或更新table_info表，根据数据库id和数据库名称
                tableInfoService.saveOrUpdate(tableInfoInsert, new LambdaUpdateWrapper<TableInfo>(TableInfo.class) {{
                    eq(TableInfo::getTableName, tableInfoInsert.getTableName());
                    eq(TableInfo::getBasicId, dataSource.getId());
                }});

                //根据数据库id和数据库名称或表名称查询
                TableInfo tableInfo = tableInfoService.selectTableInfoByName(tableInfoInsert);
                //获取数据库元数据
                DatabaseMetaData metaData = conn.getMetaData();
                //查询数据库中所有表的信息
                ResultSet rs = metaData.getTables(dataSource.getDatabaseName(), null, "%", new String[]{"TABLE"});

                while (rs.next()) {
                    String tableName = rs.getString("TABLE_NAME");//获取表名
                    String tableRemark = rs.getString("REMARKS");//获取表备注
                    //准备SQL语句，查询表中的所有数据
                    PreparedStatement preparedStatement = conn.prepareStatement(SHOW_TABLE_STATUS + " FROM `" + dataSource.getDatabaseName() + "` LIKE '" + tableName + "'");

                    ResultSet resultSet = preparedStatement.executeQuery();

                    //计算表中的行数
                    long aLong;
                    if (resultSet.next()){
                        aLong = resultSet.getLong("Rows");
                    } else {
                        aLong = 0L;
                    }
                    //构建一个新的TableInfo对象，用于插入或更新数据库中的table_info表
                    TableInfo build = TableInfo.builder()
                            .basicId(dataSource.getId())
                            .tableName(tableName)
                            //bug点，tableRemark为空，造成空指针异常
                            .tableRemark(tableRemark == null ? "" : tableRemark)// 如果表备注为空，则设置为空字符串，避免空指针异常
                            .parentId(tableInfo.getId())
                            .type("dataTable")// 表示数据表
                            .isCenter("Y") //表示中心数据库的标志
                            .updateBy(SecurityUtils.getUsername())//获取当前用户名作为更新者
                            .dataNum(aLong)// 设置数据行数
                            .updateTime(new Date())// 设置更新时间为当前时间
                            .build();
                    // 插入或更新table_info表，根据数据库id和表名
                    tableInfoService.saveOrUpdate(build, new LambdaUpdateWrapper<>(TableInfo.class) {{
                        eq(TableInfo::getTableName, build.getTableName());
                        eq(TableInfo::getBasicId, dataSource.getId());
                    }});
                    //根据数据库id和表名查询数据
                    TableInfo table = tableInfoService.selectTableInfoByName(build);

                    //创建一个线程池，用于并发同步数据
                    ExecutorService threadPool = Executors.newCachedThreadPool();

                    //提交一个任务到线程池，同步数据
                    threadPool.submit(() -> {
                        syncData(conn, dataSource.getDatabaseName(), table);
                    });
                }
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return 1;
    }

    /**
     * 可用数据
     *
     * @return 可用数据源集合
     */
    @Override
    public List<DataSourceResp> getIsUseDataSourceConfig () {
        LambdaQueryWrapper<DataSource> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(DataSource::getStatus, "0");
        return this.list(queryWrapper).stream()
                .map(DataSourceResp::dataSourceRespBuild)
                .toList();
    }

    public void syncData(Connection conn, String databaseName, TableInfo table) {
        ExecutorService threadPool = Executors.newCachedThreadPool();
        PreparedStatement ps = null;
        try {
            ps = conn.prepareStatement("    SELECT  " +
                    "    COLUMN_NAME , " +
                    "    COLUMN_COMMENT  ," +
                    "    CASE WHEN COLUMN_KEY = 'PRI' THEN '是' ELSE '否' END ," +
                    "    CASE \n" +
                    "    WHEN DATA_TYPE = 'int' THEN 'Integer' " +
                    "    WHEN DATA_TYPE = 'bigint' THEN 'Long' " +
                    "    WHEN DATA_TYPE = 'varchar' THEN 'String' " +
                    "    WHEN DATA_TYPE = 'decimal' THEN 'BigDecimal' " +
                    "    WHEN DATA_TYPE = 'tinyint' AND COLUMN_TYPE = 'tinyint(1)' THEN 'Boolean'" +
                    "    ELSE DATA_TYPE   \n" +
                    "    END , " +
                    "    DATA_TYPE ,   \n" +
                    "    COLUMN_TYPE ,  \n" +
                    "    CHARACTER_MAXIMUM_LENGTH ,  \n" +
                    "    NUMERIC_SCALE ,  \n" +
                    "    IS_NULLABLE ,  \n" +
                    "    COLUMN_DEFAULT   \n" +
                    "FROM  INFORMATION_SCHEMA.COLUMNS  WHERE  \n" +
                    "TABLE_SCHEMA = '" + databaseName + "'   \n" +
                    "AND TABLE_NAME = '" + table.getTableName() + "'");

            // 执行查询，获取结果集
            ResultSet resultSet = ps.executeQuery();
            // 遍历结果集中的每一行数据
            while (resultSet.next()) {
                // 获取列的相关信息
                String columnName = String.valueOf(resultSet.getString(1));
                String columnComment = String.valueOf(resultSet.getObject(2));
                String columnKey = String.valueOf(resultSet.getObject(3));
                // 这里应该是resultSet.getString(4)，原代码中为resultSet.getObject(4)
                String end = String.valueOf(resultSet.getObject(4));
                String dataType = String.valueOf(resultSet.getObject(5));
                // 这里应该是resultSet.getString(6)，原代码中为resultSet.getObject(6)
                String columnType = String.valueOf(resultSet.getObject(6));
                // 这里应该是resultSet.getString(7)，原代码中为resultSet.getInt(7)
                String characterMaximumLength = String.valueOf(resultSet.getInt(7));
                // 这里应该是resultSet.getString(8)，原代码中为resultSet.getInt(8)
                String NumericScale = String.valueOf(resultSet.getInt(8));
                // 这里应该是resultSet.getString(9)，原代码中为resultSet.getObject(9)
                String isNullable = String.valueOf(resultSet.getObject(9));
                String columnDefault = String.valueOf(resultSet.getObject(10));

                DataAnalysis build = DataAnalysis.builder()
                        .tableId(table.getId())
                        .columnName(columnName)
                        .columnRemark(columnComment)
                        .isPrimary("是".equals(columnKey) ? "Y" : "N")
                        .javaType(end)
                        .columnType(columnType)
                        .columnLength(characterMaximumLength)
                        .columnDecimals(NumericScale)
                        .isNull("YES".equals(isNullable) ? "Y" : "N")
                        .defaultValue(columnDefault)
                        .build();

                threadPool.submit(() -> {
                    dataAnalysisService.saveOrUpdate(build, new LambdaUpdateWrapper<DataAnalysis>() {{
                        eq(DataAnalysis::getTableId, build.getTableId());
                        eq(DataAnalysis::getColumnName, build.getColumnName());
                        eq(DataAnalysis::getColumnRemark, build.getColumnRemark());
                    }});
                });
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}






